@@ -161,7 +161,7 @@ def column_to_histo( dfcol , binmax = 32*1024, savename = "", hname = "h1", writ
     th1f = ROOT.TH1F(hname, savename, binmax, 0 , binmax)
     #for i in range(len(his)):
     #    th1f.SetBinContent( i, his[i] )
-    print("D... narr=",narr)
+    #print("D... narr=",narr)
     print(narr.shape  ,len(narr.shape) )
     print(narr.shape[0] )
     fill_hist( th1f,  his )
@@ -190,7 +190,7 @@ def save_to_tree(df2, filename):
     # convert to dict with numpy arrays
     #data = {key: df[key].values for key in ['time', 'E', 'x']}
     data = {key: df2[key].values for key in df2.columns }
-
+    print("D... columns that go to TTree:", df2.columns)
     rdf = ROOT.RDF.MakeNumpyDataFrame( data )
     print(rdf)
     outfile =  os.path.splitext(filename)[0] +"_tree.root"
@@ -200,25 +200,89 @@ def save_to_tree(df2, filename):
 #========================== READ; ANALYSIS AND DETECTION ===================
 
 #------------------------------------------------------------------------
-def pd_detect_zeroes(df):
+def pd_detect_zeroes(df, channel,  TIMEWIN_US = 25  ):
     """
+   1-get the channel only
+   2-dtus
     get number of zeroes, single zeroes, double zeroes, standalone zeroes
     -
       2nd phase - SZ and DZ - make one event from these PILEUPs
       3rd phase - create
     """
-    dfzero = df[ (df.E==0) ]
-    print(f"D... zeroes {len(dfzero)} which is  {len(dfzero)/len(df)}")
+
+    print("D... MEMORY", df.memory_usage())
+    # CHANNEL (i hope it is sorted)
+    df = df.loc[ df["ch"] == channel ]
+    #
+    print(f"D... broadening table - dt")
+    df['dtus'] = (df.time-df.time.shift())*1000*1000
+    print("D... MEMORY", df.memory_usage())
+    print(f"D... broadening table - dt")
+    df['next_E'] = (df.E.shift())
+    print("D... MEMORY", df.memory_usage())
+
+    #df['dtus'] = (df.t-df.t.shift())*1000*1000
+    #dfm = pd.concat([df, df.shift(-1).add_prefix('next_')], axis=1)
+    #print(f"D... adding dt [us]")
+    #dfm['dtus'] = (dfm.next_time-dfm.time)*1000*1000
+    # completely modified (new)  df
+
+
+    # # ---fast way and memory saving to create histogram
+    # binmax = 100
+    # narr = df['dtus'].to_numpy()
+    # print("D... len=", len(narr), narr.dtype, ".. now to histo")
+    # his,bins = np.histogram(narr,bins=binmax,range=(0.0,binmax) )
+
+
+    # #print(f"D... plotting DOUBLE the window RANGE (range={TIMEWIN_US})")
+    # #plt.plot( his, '.' )
+    # #plt.xlabel('dt [us]')
+    # #plt.ylabel(f"ch={channel}")
+    # #plt.savefig("z_time_distr.png")
+    # #sys.exit(0)
+    # #--- df is prepared
+
+
+
+    df_dz = df.loc[ ((df.E==0) & (df.next_E==0)) & (df.dtus<TIMEWIN_US) ]
+    dzeroes = len(df_dz)
+
+    df_sz = df.loc[ ((df.E!=0) & (df.next_E==0)) & (df.dtus<TIMEWIN_US) ]
+    szeroes = len(df_sz)
+
+    df_iz = df.loc[ ((df.E==0) & (df.next_E!=0)) & (df.dtus<TIMEWIN_US) ]
+    izeroes = len(df_iz)
+
+
+    print(f"D... ---------------------- START of zero detection ch={channel}")
+    df_z = df[ (df.E==0) & (df.ch==channel)]
+    zeroes = len(df_z)
+
+    # print(df) # seems to stay the same
+    print(df)
+    print("D... MEMORY", df.memory_usage())
+
+    print(f"D... total  zeroes (chan={channel}) = {zeroes} which is  {zeroes/len(df)*100:5.2f}%")
+    print(f"D... double zeroes (chan={channel}) = {dzeroes} which is  {dzeroes/len(df)*100:5.2f}%")
+    print(df_dz)
+    print(f"D... single zeroes (chan={channel}) = {szeroes} which is  {szeroes/len(df)*100:5.2f}%")
+    print(df_sz)
+    print(f"D... ilogic zeroes (chan={channel}) = {izeroes} which is  {izeroes/len(df)*100:5.2f}%")
+    print(df_iz)
+    print("D... ______________________ end of zero detection")
+
     # returns length for now
-    return len(dfzero)
+    return zeroes,dzeroes
 
 #------------------------------------------------------------------------
 
-def pd_read_table(filename):
+def pd_read_table(filename, sort = True):
     """
     READ THE ASC TABLE:   two possibilities:  4 columns OR 5 columns
     """
     df = pd.read_table( filename, names=['time','E','x','ch','y'], sep = "\s+", comment="#")
+    df['time'] = df['time']/1e+8
 
     #    df.to_hdf('run'+str(number)+'.h5',
     #              "matrix",
@@ -226,6 +290,14 @@ def pd_read_table(filename):
     #              data_columns=True,
     #              mode='w')
 
+    if sort:
+        print("D... sorting (after read_table):")
+        #print(df)
+        df = df.sort_values(by="time")
+        #print(df)
+        df.reset_index(inplace=True, drop=True)
+        #print(df)
+
     return df
 
 
